#!/bin/sh
':' //; exec "$(command -v nodejs || command -v node)" "$0" "$@"

var argv = require('minimist')(process.argv.slice(2), {
    string: 'output name service',
    boolean: 'noTest',
    alias: {
        o: 'output',
        n: 'name',
        s: 'service'
    }
});

var Path = require('path');
var fs = require('fs');
var fsUtils = require('nodejs-fs-utils');
var HBS = require('handlebars');
var P = require('bluebird');

console.log("Pagemesh CRUD Generator");

var generateCreateService = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/create.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var serviceOutput = Path.resolve(outputRoot, "services");
    fsUtils.mkdirsSync(serviceOutput);
    fs.writeFileSync(Path.join(serviceOutput, filename), result, 'utf8');
});

var generateUpdateService = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/update.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var serviceOutput = Path.resolve(outputRoot, "services");
    fsUtils.mkdirsSync(serviceOutput);
    fs.writeFileSync(Path.join(serviceOutput, filename), result, 'utf8');
});

var generateListService = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/list.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var serviceOutput = Path.resolve(outputRoot, "services");
    fsUtils.mkdirsSync(serviceOutput);
    fs.writeFileSync(Path.join(serviceOutput, filename), result, 'utf8');
});

var generateLoadService = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/load.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var serviceOutput = Path.resolve(outputRoot, "services");
    fsUtils.mkdirsSync(serviceOutput);
    fs.writeFileSync(Path.join(serviceOutput, filename), result, 'utf8');

});

var generateRemoveService = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/remove.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var serviceOutput = Path.resolve(outputRoot, "services");
    fsUtils.mkdirsSync(serviceOutput);
    fs.writeFileSync(Path.join(serviceOutput, filename), result, 'utf8');

});

var generateCreateUnitTest = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/create-unit.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var testOutput = Path.resolve(outputRoot, "test/unit");
    fsUtils.mkdirsSync(testOutput);
    fs.writeFileSync(Path.join(testOutput, filename), result, 'utf8');
});

var generateUpdateUnitTest = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/update-unit.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var testOutput = Path.resolve(outputRoot, "test/unit");
    fsUtils.mkdirsSync(testOutput);
    fs.writeFileSync(Path.join(testOutput, filename), result, 'utf8');
});

var generateListUnitTest = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/list-unit.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var testOutput = Path.resolve(outputRoot, "test/unit");
    fsUtils.mkdirsSync(testOutput);
    fs.writeFileSync(Path.join(testOutput, filename), result, 'utf8');
});

var generateLoadUnitTest = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/load-unit.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var testOutput = Path.resolve(outputRoot, "test/unit");
    fsUtils.mkdirsSync(testOutput);
    fs.writeFileSync(Path.join(testOutput, filename), result, 'utf8');
});

var generateRemoveUnitTest = P.method(function(filename, name, modelName, service, outputRoot) {
    var tmpl = HBS.compile(fs.readFileSync(Path.resolve(__dirname, "tmpl/remove-unit.js.hbs"), 'utf8'));
    var result = tmpl({
        modelName: modelName,
        service: service,
        target: name,
        name: name
    });

    var testOutput = Path.resolve(outputRoot, "test/unit");
    fsUtils.mkdirsSync(testOutput);
    fs.writeFileSync(Path.join(testOutput, filename), result, 'utf8');
});

var output = Path.resolve(process.cwd(), argv.output);
var modelName = argv.name.toLowerCase();

// Generate services
return P.join(
    generateCreateService('create-'+modelName+".service.js", modelName, argv.name, argv.service, output),
    generateUpdateService('update-'+modelName+".service.js", modelName, argv.name, argv.service, output),
    generateListService('list-'+modelName+".service.js", modelName, argv.name, argv.service, output),
    generateLoadService('load-'+modelName+".service.js", modelName, argv.name, argv.service, output),
    generateRemoveService('remove-'+modelName+".service.js", modelName, argv.name, argv.service, output)
).then(function() {
    if(!argv.noTest) {
        console.log("Generating unit test for all CRUD services");
        return P.join(
            generateCreateUnitTest('create-'+modelName+".spec.js", modelName, argv.name, argv.service, output),
            generateUpdateUnitTest('update-'+modelName+".spec.js", modelName, argv.name, argv.service, output),
            generateListUnitTest('list-'+modelName+".spec.js", modelName, argv.name, argv.service, output),
            generateLoadUnitTest('load-'+modelName+".spec.js", modelName, argv.name, argv.service, output),
            generateRemoveUnitTest('remove-'+modelName+".spec.js", modelName, argv.name, argv.service, output)
        );
    }
}).catch(function(err) {
    console.error("Unable to generate services", err);
});
